#ifndef JSOBJC
#define JSOBJC
#endif
#ifndef JSFILE
#define JSFILE
#endif
#ifndef pid_t
#ifndef Rhapsody
#define pid_t int
#endif
#endif
#import "jsapi.h"
#ifdef WIN32
#ifdef __TOPLEVEL_NEXT_COMMON_INCLUDE__
#undef __TOPLEVEL_NEXT_COMMON_INCLUDE__
#include <next_common_undefines.h>
#endif
#else
#ifdef id
#undef id
#endif
#endif
#import <Foundation/Foundation.h>
#import <itkObjCRuntime/itkObjCRuntimeTypes.h>
#import <itkJavaScript/ITKJSVal.h>
#import <itkJavaScript/ITKJSProperties.h>

typedef struct tagJSObjCSavedState {
    JSBool		destroy;
    void		*next;
    JSErrorReporter	errorReporter;
    JSExceptionState    *exceptionState;
    NSException		*exception;
    NSMutableArray     	*warnings;
} JSObjCSavedState;

typedef enum tagJSObjCTeachFlags {
    JSObjCInstanceMethod = 0x1,	/* teach an instance method to a class */
    JSObjCTakesSelfArg = 0x2,	/* JS function has "self" arg */
    JSObjCTakesCmdArg = 0x4,	/* JS function has "_cmd" arg */
    JSObjCTakesVarArgs = 0x8	/* method takes variable argument list */
} JSObjCTeachFlags;

extern NSString *JSObjCJSErrorException;

JSBool
JSObjC_Init (JSContext *cx, size_t stacksize);

void
JSObjC_ReportNSException (JSContext *cx, NSException *localException);

NSException *
JSObjC_NSExceptionFromJSException(JSContext *cx, jsval exn);

void
JSObjC_EnterJS (JSContext *cx, JSObjCSavedState *state);

void
JSObjC_LeaveJS (JSContext *cx, JSObjCSavedState *state, JSBool ok);

void
JSObjC_EventuallyDestroyContext (JSContext *cx);

JSScript *
JSObjC_PreprocessAndCompileFile (JSContext *cx, JSObject *obj,
                                 const char *filename);

JSBool
JSObjC_PrecompileFile (JSContext *cx, JSObject *obj, const char *filename,
                       const char *outfile);

JSBool
JSObjC_LoadPrecompiledFile (JSContext *cx, JSObject *obj, const char *filename);

JSBool
JSObjC_ImportFile (JSContext *cx, JSObject *obj, const char *filename,
		   const char *from, JSBool system, JSBool once);

JSBool
JSObjC_Teach (JSContext *cx, id obj, SEL sel, const char *types,
	      JSObjCTeachFlags flags, JSObject *funobj);

JSBool
JSObjC_Unteach (JSContext *cx, id anObject, SEL sel, JSObjCTeachFlags flags,
		int *deleteCount);

JSBool
JSObjC_DeclareTypedef (JSContext *cx, JSObject *obj,
		       const char *name, const char *type);

const char *
JSObjC_LookupTypedef (JSContext *cx, const char *name);

JSBool
JSObjC_DeclareClass (JSContext *cx, JSObject *obj, const char *name);

void *
JSObjC_DeclareVariable (JSContext *cx, JSObject *obj, const char *name,
			const char *type, void *addr);

JSBool
JSObjC_DeclareLocalVariable (JSContext *cx, JSFunction *fun, const char *name,
			     const char *type);

JSBool
JSObjC_DeclareFunction (JSContext *cx, JSObject *obj, const char *name,
			const char *types);

JSBool
JSObjC_NewIdValue (JSContext *cx, id anObject, ItkObjCPromote promote,
                   jsval *vp);

JSBool
JSObjC_ValueToId (JSContext *cx, jsval *vp, id *idp);

JSBool
JSObjC_NewSelValue (JSContext *cx, SEL sel, jsval *vp);

JSBool
JSObjC_ValueToSel (JSContext *cx, jsval *vp, SEL *selp);

JSBool
JSObjC_NewNumberValue (JSContext *cx, void *p, const char *type, jsval *vp);

JSBool
JSObjC_ValueToNumber (JSContext *cx, jsval *vp, const char *type, void *p);

JSBool
JSObjC_NewLongLongValue (JSContext *cx, long long ll, jsval *vp);

JSBool
JSObjC_ValueToLongLong (JSContext *cx, jsval *vp, long long *llp);

JSBool
JSObjC_NewUnsignedLongLongValue (JSContext *cx, unsigned long long ll,
                                 jsval *vp);

JSBool
JSObjC_ValueToUnsignedLongLong (JSContext *cx, jsval *vp,
                                unsigned long long *llp);

JSBool
JSObjC_NewPointerValue (JSContext *cx, const char *type, void *addr,
                        jsint length, jsval *vp);

JSBool
JSObjC_ValueToPointer (JSContext *cx, jsval *vp,
                       const char **typep, void **addrp);

JSBool
JSObjC_CastValueToPointer (JSContext *cx, jsval *vp,
                           const char **typep, void **addrp);

JSBool
JSObjC_ValueToArray (JSContext *cx, jsval *vp, const char *type, jsint length,
		     void *arrp);

JSBool
JSObjC_NewFunctionValue (JSContext *cx, const char *name, void *addr,
                         const char *types, jsval *vp);

JSBool
JSObjC_NewStructValue (JSContext *cx, const char *type, void *addr, jsval *vp);

JSBool
JSObjC_NewStructValueCopy (JSContext *cx, const char *type, void *addr,
                           jsval *vp);

JSBool
JSObjC_ValueToStruct (JSContext *cx, jsval *vp, const char *type, void *addr);

JSBool
JSObjC_NewUnionValue (JSContext *cx, const char *type, void *addr, jsval *vp);

JSBool
JSObjC_NewUnionValueCopy (JSContext *cx, const char *type, void *addr,
                          jsval *vp);

JSBool
JSObjC_ValueToUnion (JSContext *cx, jsval *vp, const char *type, void *addr);

JSBool
JSObjC_NewBinaryValue (JSContext *cx, void *binary, const char *type,
                       ItkObjCPromote promote, jsval *vp);

JSBool
JSObjC_ValueToBinary (JSContext *cx, jsval *vp, const char *type,
		      ItkObjCPromote promote, void *binary);

JSBool
JSObjC_Cast (JSContext *cx, const char *type, jsval *vp);

static inline JSClass *
JSObjC_GetClass(JSContext *cx, JSObject *obj)
{
#ifdef JS_THREADSAFE
    return JS_GetClass(cx, obj);
#else
    return JS_GetClass(obj);
#endif
}
