#import <AppKit/AppKit.h>
#import <Foundation/Foundation.h>
#import "ITKScriptingLanguage.h"
#import "ITKValue.h"

#define FROB(constant) extern NSString *constant;
FROB(ITKInterpVariableDidChangeNotification)
FROB(ITKInterpKey)
FROB(ITKVariableKey)
FROB(ITKGlobalKey)
FROB(ITKOldValueKey)
FROB(ITKNewValueKey)
#undef FROB

@class ITKCommandTextView;

// ITKInterp is an abstract superclass for interpreter objects.

@interface ITKInterp : NSObject
{
    NSString		*initFile;
    ITKCommandTextView	*commandText;
    NSMapTable		*timedEvals, *watchedVars;
    unsigned		mainInterpProxy : 1;
    unsigned		browserLoaded : 1;
    unsigned		javaLoaded : 1;
    unsigned		reserved : 29;
}

// The first interpreter created by the application (nil if none exists yet)

+ (ITKInterp *) mainInterp;

// Access methods

- (NSString *) initFile;
- (void) setInitFile: (NSString *) anInitFile;
- (BOOL) isMainInterpProxy;
- (void) setMainInterpProxy: (BOOL) flag;

// Generic interpreter functionality

- (ITKValue *) sourceFile: (NSString *) file;

- (ITKValue *) evalString: (NSString *) string global: (BOOL) global;
- (ITKValue *) evalValue: (ITKValue *) value global: (BOOL) global;

- (ITKValue *) setVariable: (NSString *) var
		   toValue: (ITKValue *) value
		    global: (BOOL) global;

- (ITKValue *) valueOfVariable: (NSString *) var global: (BOOL) global;

- (void) setPostsChangedNotifications: (BOOL) flag
			  forVariable: (NSString *) var
			       global: (BOOL) global;
		      
- (void) _postChangedNotificationWithUserInfo: (NSMutableDictionary *) userInfo;

@end

@interface ITKInterp (AbstractMethods)

// This category is implemented only by concrete subclasses.

- (id <ITKScriptingLanguage>) scriptingLanguage;
- (void) unsetVariable: (NSString *) var global: (BOOL) global;
- (void *) nativeInterp;

- (void *) _sourceFile: (NSString *) file;
- (void *) _eval: (void *) script global: (BOOL) global;
- (void *) _setVariable: (NSString *) var
		toValue: (void *) value
		 global: (BOOL) global;
- (void *) _valueOfVariable: (NSString *) var global: (BOOL) global;
- (void) _setPostsChangedNotifications: (BOOL) flag
			   forVariable: (NSString *) var
				global: (BOOL) global
			      userInfo: (NSMutableDictionary *) userInfo;
- (void) _enableJavaSupport;

@end

@interface ITKInterp (JavaSupport)

// Called as a part of subclasses' initialization. Might call enableJavaSupport,
// depending on preference settings.

- (void) enableJavaSupportIfNeeded;

// Will pull in the Java framework and enable language-specific Java support.

- (void) enableJavaSupport;

@end

@interface ITKInterp (TimedEvals)

// All these methods return some token object that can be passed to
// removeTimedEval: to cancel the request. The modes argument defaults to
// [NSArray arrayWithObject: NSDefaultRunLoopMode] if omitted.

- evalString: (NSString *) string afterDelay: (NSTimeInterval) delay;
- evalString: (NSString *) string afterDelay: (NSTimeInterval) delay
     inModes: (NSArray *) modes;

- evalString: (NSString *) string withInterval: (NSTimeInterval) interval;
- evalString: (NSString *) string withInterval: (NSTimeInterval) interval
     inModes: (NSArray *) modes;

- evalString: (NSString *) string afterThisEventOrdered: (unsigned) order;
- evalString: (NSString *) string afterThisEventOrdered: (unsigned) order
     inModes: (NSArray *) modes;

- evalString: (NSString *) string afterEveryEventOrdered: (unsigned) order;
- evalString: (NSString *) string afterEveryEventOrdered: (unsigned) order
     inModes: (NSArray *) modes;

- evalStringWhenDeleted: (NSString *) string;

- (void) removeTimedEval: token;

- (void) _invalidateTimedEval: token;

@end

@interface ITKInterp (CommandWindow)

- (ITKCommandTextView *) commandTextView;
- (NSWindow *) commandWindow;
- (void) showCommandWindow: sender;
- (void) clearCommandWindow: sender;
- (void) saveCommands: sender;
- (NSString *) commandWindowTitle;
- (NSString *) commandPrompt;

@end

@interface ITKInterp (OnlineHelp)

- (BOOL) showHelpForCommand: (NSString *) command;
- (void) showContextHelp: sender;

@end

