#import <Foundation/Foundation.h>
#import "ITKScriptingLanguage.h"

// ITKValue is an abstract superclass for value objects that are wrappers for
// the "native representation" of values in a certain scripting language.

@interface ITKValue : NSObject

// Create a value object from the given native representation, which is an
// opaque handle as far as ITKValue is concerned.

+ (ITKValue *) valueWithNativeRep: (void *) native;

+ (ITKValue *) valueWithString: (NSString *) string;

// Get a value object that represents a "sequence" (subject to interpretation
// by the concrete scripting language) composed of the values in array, which
// have to be of the same class this message is sent to.

+ (ITKValue *) valueWithArray: (NSArray *) array;

+ (ITKValue *) valueWithBytes: (void *) bytes objCType: (const char *) type;

// Get a value object that represents a pointer to the given type.

+ (ITKValue *) valueWithPointer: (void *) addr toObjCType: (const char *) type;

#define FROB(creator,accessor,type)					       \
+ (ITKValue *) creator (type) value;					       \
- (type) accessor;

FROB(valueWithObject:,		objectValue,		id)
FROB(valueWithPointer:,		pointerValue,		void *)
FROB(valueWithChar:,		charValue,		char)
FROB(valueWithUnsignedChar:,	unsignedCharValue,	unsigned char)
FROB(valueWithShort:,		shortValue,		short)
FROB(valueWithUnsignedShort:,	unsignedShortValue,	unsigned short)
FROB(valueWithInt:,		intValue,		int)
FROB(valueWithUnsignedInt:,	unsignedIntValue,	unsigned int)
FROB(valueWithLong:,		longValue,		long)
FROB(valueWithUnsignedLong:,	unsignedLongValue,	unsigned long)
FROB(valueWithLongLong:,	longLongValue,		long long)
FROB(valueWithUnsignedLongLong:,unsignedLongLongValue,	unsigned long long)
FROB(valueWithFloat:,		floatValue,		float)
FROB(valueWithDouble:,		doubleValue,		double)
FROB(valueWithBool:,		boolValue,		BOOL)
#undef FROB

@end

@interface ITKValue (AbstractMethods)

// This category is implemented only by concrete subclasses.

+ (ITKValue *) nullValue;

- (BOOL) isNull;

- (ITKValue *) initWithNativeRep: (void *) native;
- (ITKValue *) initWithString: (NSString *) string;
- (ITKValue *) initWithArray: (NSArray *) array;
- (ITKValue *) initWithBytes: (void *) value objCType: (const char *) type;

- (void *) nativeRep;
- (NSString *) source;
- (NSString *) stringValue;
- (NSArray *) arrayValue;
- (void) getValue: (void *) value objCType: (const char *) type;

- (id <ITKScriptingLanguage>) scriptingLanguage;

@end
