#--------------------------------------------------------------------------
#  FileBrowser.nib/Interpreter.tcl
#  Copyright (c) 1997-98, AAA+ Software (info@aaa-plus.com)
#  All rights reserved.
#  Author: W. Pitzeier
#
#  You may freely copy, distribute and reuse the code in this example.
#  AAA+ disclaims any warranty of any kind, expressed or implied,
#  as to its fitness for any particular use.
#--------------------------------------------------------------------------


# teach the two skills needed for a passive browser delegate
objc:teach $self {

    # skill that gets called each time a new cell is displayed
    - void browser: sender willDisplayCell: cell atRow: {int row} column: {int column} {
        global fileManager
		if {$column == 0} {
			$cell setStringValue: [@ [lindex $::volumes $row]]
			return
		}
		set path [$sender pathToColumn: $column] 
		set file [[[$fileManager directoryContentsAtPath: $path] sortedArrayUsingSelector: caseInsensitiveCompare: ] objectAtIndex: $row]
        $cell setStringValue: $file
        if {[$fileManager directoryContentsAtPath: [$path stringByAppendingPathComponent: $file]] == "nil"} {
            $cell setLeaf: [YES]
        } else {
            $cell setLeaf: [NO]
        }
    }
    
    # skill that gets called each time a new column is displayed
    - int browser: sender numberOfRowsInColumn: {int column} {
        global fileManager
		if {$column == 0} {
			set count [llength $::volumes]
		} else {
			set path [$sender pathToColumn: $column] 
			set count [[$fileManager directoryContentsAtPath: $path] count]
		}
		return $count
    }
}


objc:teach $browser {

	# removes the leading /
	- pathToColumn: {int column} {
		return [[former pathToColumn: $column] substringFromIndex: 1]
	}

    # Needed for supporting filename services (such as "Joy Snooping")
    - validRequestorForSendType: sendType returnType: returnType {
        if {[$sendType isEqualToString: [NSFilenamesPboardType]]} {
            return $self
        }
        return nil
    }
    
    - BOOL writeSelectionToPasteboard: pboard types: types {
        $pboard declareTypes: [NSArray arrayWithObject: [NSFilenamesPboardType]] owner: nil
        $pboard setPropertyList: [NSArray arrayWithObject: [$self path]] forType: [NSFilenamesPboardType] 
    }
    
    - void action: sender {
        global icon workspace
        set fileicon [$workspace iconForFile: [$self path]]
        if {$fileicon == "nil"} {
            $icon setEnabled: [NO]
        } else {
            $icon setEnabled: [YES]
        }
        $icon setImage: $fileicon
    }
}


objc:teach $icon {

    - void action: sender {
        global workspace browser
        $workspace openFile: [$browser path]
    }
}

$browser setDelegate: $self
# for window resizing:
# if the window width gets to small,
# reduze the number of displayed columns
$browser setMinColumnWidth: 135
# if the window width gets larger, display more columns
$browser setMaxVisibleColumns: 5
set fileManager [NSFileManager defaultManager]
set workspace [NSWorkspace sharedWorkspace]
set volumes [file volume]

$NSApp registerServicesMenuSendTypes: [NSArray arrayWithObject: [NSFilenamesPboardType]] returnTypes: nil

$infoPanel center
$window center
$window makeKeyAndOrderFront: $self
