/* HTTP Source Browser example
 *
 * This program shows how easy it is to use Java and Objective-C together.
 * In this example, Java has been used for the string manipulation and the
 * socket interaction, and Objective-C has been used to communicate with the
 * window system.
 *
 * No attempt has been made to render the HTML. Firstly, the URL that the
 * user has typed is decomposed into a hostname and a file path. If no '/'
 * character is present in the URL, then it is assumed to be just a hostname,
 * in which case the root document is required. Otherwise, the host and
 * pathname are extracted. In any case, the results are put in variables 'host'
 * and 'file'. Notice that although the variables are first introduced in
 * a lower scope (the 'if' statement), they are still accessible within the
 * procedure. All variables are global to procedures, not to lexically-scoped
 * blocks as in C, Java, and most other traditional compiled languages.
 *
 * Secondly, the socket is opened. Then, the byte streams that are associated
 * with the socket (input and output) are firstly translated into character
 * streams, and then wrapped in a buffer. This is all possible with Java's
 * advanced class library, now accessible from anywhere within a Joy script.
 *
 * The HTTP request is written to the buffered stream connected with the
 * socket, and the buffer is flushed. The results are printed to the output
 * text view: firstly all text therein is selected, and secondly one by one
 * the lines back from the server are added. The first line to be written
 * overwrites the whole of the old contents of the buffer, by virtue that it
 * was selected.
 *
 * Adrian Smith
 * AAA+ Software
 * 18 Jan 1999
 */

[self enableJavaSupport]

url["action:"] = function() {
    /* Get the URL the user's requested (Objective-C syntax)*/
    http = new java.lang.String([url stringValue]);
    
    /* Decompose into hostname and doc name (C character manipulation) */
    slashPoint = http.indexOf('/');
    if (slashPoint == -1) {
	host = http; file = "/";
    } else {
        host = http.substring(0, slashPoint);
        file = http.substring(slashPoint);
    }
        
    /* Open socket (all sockets done with Java syntax and libraries) */
    s = new java.net.Socket(host, 80);

    /* writing to the remote computer */
    o = new java.io.OutputStreamWriter(s.getOutputStream());
    bw = new java.io.BufferedWriter(o);
    
    /* reading from the remote computer */
    i = new java.io.InputStreamReader(s.getInputStream());
    br = new java.io.BufferedReader(i);
    
    /* send initial message to web server */
    bw.write("GET "+file+" HTTP/1.0");
    bw.newLine(); bw.newLine();
    bw.flush();

    /* Display it in the text box */
    [html selectAll:self];
    while (line = br.readLine())
        [html insertText:line + "\n"];
}
